import java.awt.Color;
public class Driver implements Tickable{
    private static final double SM = 1.98855*Math.pow(10,30);
    private static final double SR = 696342000;
    private static final double EM = 5.97219*Math.pow(10,24);//5.97219 x 24
    private static final double ER = 6371000;
    private static final double MM = 6.4185*Math.pow(10,23);
    private static final double MR = 3389500;
    private static final double JM = 1.8986*Math.pow(10,27);
    private static final double JR = 69911000;
    private static final Vector still = new Vector(0,0,0);//Do not use for position, messes things up!
    private static final Vector origin = new Vector(0.001,0.001,0.001);
    private static final Vector earthPos = new Vector(149598261000.0+ER+SR,0,0);//149598261000
    private static final Vector earthVel = new Vector(0,29780,0);//29780
    private static final Vector marsPos = new Vector(227939100000.0,0,0);
    private static final Vector marsVel = new Vector(0,24077,0);
    private static final Vector jovPos = new Vector(778547200000.0,0,0);//778547200000
    private static final Vector jovVel = new Vector(0,13070,0);//13070
    private static Body[] system = new Body[4];
    private static ShapeNode[] circles = new ShapeNode[system.length];
    private static TextNode days;
    private static double t = 0;
    private static int day = 0;
    private static double xmax=1;
    private static double xmin=-1;
    private static double ymax=1;
    private static double ymin=-1;
    private static PlayScreen graph;
    private static double keepT = 0;
    public Driver(PlayScreen ps){
        graph = ps;
    }
    public static void main(String[] args){
        system[0] = new Body(origin,still,SM,SR);//The sun, at the center
        system[1] = new Body(earthPos,earthVel,EM,ER);//The earth
        //system[2] = new Body(moonPos,moonVel,MM,MR);//Mars
        double tInc = 1.0;
        double t = 0;
        for(int i = 0;/*t<300000000.0*/;){
            //System.out.println(d+" ("+i+")");
            for(Body b:system){
                for(Body r:system){
                    if(!b.equals(r)){
                        if(Body.intersects(b,r)){
                            if(r.mass()>b.mass()){
                                b.setVel(r.vel());
                            }
                            else{
                                r.setVel(b.vel());
                            }
                        }
                        b.pull(tInc,r);
                    }
                }
            }
            for(Body b:system){
                b.move(tInc);
            }
            System.out.println(Body.distance(system[1],system[2]));
        }
    }
    public void init(double tInc){
        keepT = tInc;
        graph.addNode(Color.WHITE,-1,-10,0,10,0,true);
        graph.addNode(Color.WHITE,-1,0,-10,0,10,true);
        system[0] = new Body(origin,still,SM,SR);//The sun, at the center
        circles[0] = graph.addNode(Color.YELLOW,0,0,0,15);
        system[1] = new Body(earthPos,earthVel,EM,ER);//The earth
        circles[1] = graph.addNode(Color.BLUE,0,0,0,5);
        system[2] = new Body(marsPos,marsVel,MM,MR);//MARS
        circles[2] = graph.addNode(Color.RED,0,0,0,5);
        system[3] = new Body(jovPos,jovVel,JM,JR);//JUPITER
        circles[3] = graph.addNode(Color.ORANGE,0,0,0,7);
        days = graph.addNode(Color.WHITE,"Day 0",1,(int)-graph.getSize().getWidth()/2+10,(int)-graph.getSize().getHeight()/2+20);
        Ticker calcTime = new Ticker(this,55);
        calcTime.start(1);//time between calculation groups
    }
    public void update(double tInc){
        //System.out.println(Vector.diff(system[1].pos(),system[2].pos()).x());
        for(int j = 0;j<1000;j++){
            for(Body b:system){
                for(Body r:system){
                    if(!b.equals(r)){
                        b.pull(tInc,r);
                    }
                }
            }
            for(Body b:system){
                b.move(tInc);
            }
            for(Body b:system){
                if(b.pos().x()>xmax){
                    xmax = b.pos().x()*1.2;
                }
                if(b.pos().y()>ymax){
                    ymax = b.pos().y()*1.2;
                }
                if(b.pos().x()<xmin){
                    xmin = b.pos().x()*1.2;
                }
                if(b.pos().y()<ymin){
                    ymin = b.pos().y()*1.2;
                }
            }
            int[][] pos = graphDims((int)graph.getSize().getWidth(),(int)graph.getSize().getHeight());
            for(int i = 0; i<circles.length;i++){
                circles[i].setPos(pos[i][0],pos[i][1]);
            }
        }
        tInc(tInc*1000);
    }
    public void tInc(double step){
        t += step;
        while(t>86400.0){
            day += 1;
            t -= 86400.0;
        }
        days.setText("Day "+day);
        days.setPos((int)-graph.getSize().getWidth()/2+10,(int)-graph.getSize().getHeight()/2+20);
    }
    public void onTick(int id,int iteration){
        update(keepT);//tInc in seconds.
    }
    public static int[][] graphDims(int x, int y){
        int[][] r = new int[system.length][2];
        double scl = scale(x,y);
        for(int i = 0;i<r.length;i++){
            r[i][0] = (int)(scl*system[i].pos().x());
            r[i][1] = (int)(scl*system[i].pos().y());
        }
        return r;
    }
    public static double sysWidth(){
        if(-xmin>xmax){
            return -2*xmin;
        }
        else{
            return 2*xmax;
        }
    }
    public static double sysHeight(){
        if(-ymin>ymax){
            return -2*ymin;
        }
        else{
            return 2*ymax;
        }
    }
    private static double scale(int x, int y){
        double xscl = x/sysWidth();
        double yscl = y/sysHeight();
        if(xscl<yscl){
            return xscl;
        }
        else{
            return yscl;
        }
    }
}
